from douyin.config import hot_music_url
from douyin.structures import HotMusic
from douyin.utils import fetch
from douyin.utils.common import parse_datetime
from douyin.utils.tranform import data_to_music


def music():
    """
    get hot music result
    :return: HotMusic object
    """
    result = fetch(hot_music_url)
    # process json data
    datetime = parse_datetime(result.get('active_time'))
    # video_list = result.get('music_list', [])
    musics = []
    music_list = result.get('music_list', [])
    for item in music_list:
        music = data_to_music(item.get('music_info', {}))
        music.hot_count = item.get('hot_value')
        musics.append(music)
        # construct HotMusic object and return
    return HotMusic(datetime=datetime, data=musics)



import requests
from retrying import retry

from douyin.config import retry_max_number, retry_min_random_wait, retry_max_random_wait, fetch_timeout


def need_retry(exception):
    """
    need to retry
    :param exception:
    :return:
    """
    result = isinstance(exception, (requests.ConnectionError, requests.ReadTimeout))
    if result:
        print('Exception', type(exception), 'occurred, retrying...')
    return result


def fetch(url, **kwargs):
    """
    warp _fetch method
    :param url: fetch url
    :param kwargs: other requests params
    :return: result of _fetch
    """
    
    @retry(stop_max_attempt_number=retry_max_number, wait_random_min=retry_min_random_wait,
           wait_random_max=retry_max_random_wait, retry_on_exception=need_retry)
    def _fetch(url, **kwargs):
        """
        fetch api response
        :param url: fetch url
        :param kwargs: other requests params
        :return: json of response
        """
        kwargs.update({'verify': False})
        kwargs.update({'timeout': fetch_timeout})
        response = requests.get(url, **kwargs)
        if response.status_code != 200:
            raise requests.ConnectionError('Expected status code 200, but got {}'.format(response.status_code))
        return response.json()
    
    try:
        result = _fetch(url, **kwargs)
        return result
    # give up retrying
    except (requests.ConnectionError, requests.ReadTimeout):
        return {}



from douyin.config import hot_energy_url
from douyin.structures import HotEnergy
from douyin.utils import fetch
from douyin.utils.common import parse_datetime
from douyin.utils.tranform import data_to_video


def energy():
    """
    get hot energy result
    :return: HotEnergy object
    """
    result = fetch(hot_energy_url)
    # process json data
    datetime = parse_datetime(result.get('active_time'))
    video_list = result.get('aweme_list', [])
    videos = []
    for item in video_list:
        video = data_to_video(item.get('aweme_info'))
        video.hot_count = item.get('hot_value')
        videos.append(video)
    # construct HotEnergy object and return
    return HotEnergy(datetime=datetime, data=videos)



import datetime


class Base():
    
    def json(self):
        """
        object to json
        :return:
        """
        from douyin.structures import Video, Music, User, Topic, Address
        # transfer object to dict
        d = self.__dict__
        # iterate every attribute
        for k, v in d.items():
            if not v:
                continue
            # recurrent call json method
            if isinstance(v, (Video, Music, User, Topic, Address)):
                d[k] = v.json()
            # to string
            if isinstance(v, datetime.datetime):
                d[k] = str(v)
        return d



from douyin.structures import Base


class User(Base):
    
    def __init__(self, **kwargs):
        super().__init__()
        self.id = kwargs.get('id')
        self.gender = kwargs.get('gender')
        self.name = kwargs.get('name')
        self.create_time = kwargs.get('create_time')
        self.birthday = kwargs.get('birthday')
        self.sign = kwargs.get('sign')
        self.alias = kwargs.get('alias')
        self.avatar = kwargs.get('avatar')
        self.verify = kwargs.get('verify')
        self.verify_info = kwargs.get('verify_info')
    
    def __repr__(self):
        """
        user to str
        :return:
        """
        return '<User: <%s, %s>>' % (self.alias, self.name)



import collections

HotSearch = collections.namedtuple('HotSearch', ['datetime', 'data'])
HotVideo = collections.namedtuple('HotVideo', ['datetime', 'data'])
HotEnergy = collections.namedtuple('HotVideo', ['datetime', 'data'])
HotMusic = collections.namedtuple('HotMusic', ['datetime', 'data'])
HotTrend = collections.namedtuple('HotTrend', ['datetime', 'data', 'count', 'offset'])


import dateparser


def parse_datetime(string):
    """
    parse string to datetime safely
    :param string: str to parse
    :return: datetime
    """
    if not string:
        return None
    return dateparser.parse(str(string))


def first(array):
    """
    get first element of list or None
    :param array:
    :return:
    """
    if isinstance(array, list) and len(array) > 1:
        return array[0]
    return None



from douyin.structures import Base
from douyin.utils.fetch import fetch
from douyin.config import music2video_url, common_headers


class Music(Base):
    
    def __init__(self, **kwargs):
        """
        music init args
        :param kwargs:
        """
        super().__init__()
        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.cover_url = kwargs.get('cover_url')
        self.play_url = kwargs.get('play_url')
        self.owner_id = kwargs.get('owner_id')
        self.owner_name = kwargs.get('owner_name')
        self.hot_count = kwargs.get('hot_count')
    
    def __repr__(self):
        """
        music to str
        :return:
        """
        return '<Music: <%s, %s>>' % (self.id, self.name)
    
    def videos(self, max=None):
        """
        get videos of topic
        :return:
        """
        if max and not isinstance(max, int):
            raise RuntimeError('`max` param must be int')
        from douyin.utils.tranform import data_to_video
        query = {
            'device_id': '33333333',
            'music_id': self.id,
            'count': '18',
        }
        offset, count = 0, 0
        while True:
            # define cursor
            query['cursor'] = str(offset)
            print(query)
            result = fetch(music2video_url, params=query, headers=common_headers, verify=False)
            print(result)
            aweme_list = result.get('aweme_list', [])
            for item in aweme_list:
                video = data_to_video(item)
                count += 1
                yield video
                if max and count >= max:
                    break
            # next page
            if result.get('has_more'):
                offset += 18
            else:
                break
